package psi.ip.gui;

import javax.swing.JOptionPane;
import psi.ip.net.IPv4Packet;

/**
 * Hlavni okno aplikace.
 */
public class MainWindowFrame extends javax.swing.JFrame {

	Controller controller;

	/**
	 * Vytvori hlavni okno aplikace.
	 * @param controller Instance tridy rozhrani mezi sitovou a prezentacni vrstvou.
	 */
    public MainWindowFrame(Controller controller) {
		this.controller = controller;
        initComponents();
		packetsTable.setModel(controller.getPacketsTableModel());
		setDevices();
		controller.setPacketsCount(Integer.parseInt(countSpinner.getValue().toString()));
    }

	/**
	 * Aktualizuje zobrazeni tabulky se zachycenymi pakety.
	 */
	public void refreshTable() {
		packetsTable.revalidate();
		packetsTable.repaint();
	}

	/**
	 * Nastavi combo box se seznamem sitovych rozhrani.
	 */
	private void setDevices() {
		int i = 0;
		for (String d : controller.getDevices()) {
			devicesComboBox.addItem(i++ + " " + d);
		}

		if (i > 0) {
			devicesComboBox.setEnabled(true);
		}

		controller.setDevice(devicesComboBox.getSelectedIndex());
	}

	@Override
	public void setVisible(boolean b) {
		super.setVisible(b);
//		devicesComboBox.setSelectedIndex(1);
	}

	/**
	 * Povoli/zakaze ovladaci prvky v zavislosti na tom, zda se zachytava, ci ne.
	 * @param value <code>true</code>, pokud se zachytava.
	 */
	public void setIsCapturing(boolean value) {
		startButton.setEnabled(!value);
		stopButton.setEnabled(value);
		devicesComboBox.setEnabled(!value);
		countSpinner.setEnabled(!value);
		tableSizeSpinner.setEnabled(!value);
		clearButton.setEnabled(!value);
	}

	/**
	 * Zpracuje a zobrazi informace o paketu, je-li ten vybran v tabulce s pakety.
	 */
	private void rowSelected() {
		int row = packetsTable.getSelectedRow();

		if (row < 0) {
			return;
		}

		IPv4Packet packet = controller.getPacket(row);

		phbTextField.setText("Unknown");
		classTextField.setText("");
		dropTextField.setText("");

		versionTextField.setText(String.valueOf(packet.getVersion()));
		ttlTextField.setText(String.valueOf(packet.getTimeToLive()));
		headerLengthTextField.setText(String.valueOf(packet.getHeaderLength()));
		totalLengthTextField.setText(String.valueOf(packet.getTotalLength()));
		crcTextField.setText("0x" + Integer.toHexString(packet.getHeaderCheckSum()));
		offsetTextField.setText(String.valueOf(packet.getFragmentOffset()));

		reservedCheckBox.setSelected(packet.getReserved());
		moreFragmentsCheckBox.setSelected(packet.getMoreFragments());
		dontFragmentCheckBox.setSelected(packet.getDontFragment());

		byte tos = packet.getTypeOfService();

		int precedence = (tos & 0xE0) >> 5;
		int dtr = (tos & 0x1C) >> 2;
		boolean d = (dtr & 0x4) != 0;
		boolean t = (dtr & 0x2) != 0;
		boolean r = (dtr & 0x1) != 0;

		boolean ect = (tos & 0x2) != 0;
		boolean ce = (tos & 0x1) != 0;

		switch (precedence) {
			case 0:
				precedenceTextField.setText("Routine");
				break;

			case 1:
				precedenceTextField.setText("Priority");
				break;

			case 2:
				precedenceTextField.setText("Immediate");
				break;

			case 3:
				precedenceTextField.setText("Flash");
				break;

			case 4:
				precedenceTextField.setText("Flash Override");
				break;

			case 5:
				precedenceTextField.setText("CRITIC/ECP");
				break;

			case 6:
				precedenceTextField.setText("Internetwork Control");
				break;

			case 7:
				precedenceTextField.setText("Network Control");
				break;
		}

		if (dtr == 0) {
			if (precedence == 0) {
				phbTextField.setText("Default");
			} else {
				phbTextField.setText("Class Selector");
				classTextField.setText(String.valueOf(precedence));
			}
			phbTextField.setText(precedence == 0 ? "Default" : "Class Selector");
		} else if (!r && precedence < 5) {
			phbTextField.setText("Assured Forwarding");
			classTextField.setText(String.valueOf(precedence));
			switch (dtr) {
				case 2:
					dropTextField.setText("Low");
					break;

				case 4:
					dropTextField.setText("Medium");
					break;

				case 6:
					dropTextField.setText("High");
					break;
			}
		} else if (!r && precedence == 5) {
			phbTextField.setText("Expedited Forwarding");
		}

		lowDelayCheckBox.setSelected(d);
		highThroughputCheckBox.setSelected(t);
		highReliabilityCheckBox.setSelected(r);
		ectCheckBox.setSelected(ect);
		ceCheckBox.setSelected(ce);


		dataTextArea.setText(Parser.parseData(packet.getData()) + "\n"+ Parser.parseOptions(packet.getOptions()));
		dataTextArea.setCaretPosition(0);
	}

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jScrollPane1 = new javax.swing.JScrollPane();
        packetsTable = new javax.swing.JTable();
        jToolBar1 = new javax.swing.JToolBar();
        startButton = new javax.swing.JButton();
        stopButton = new javax.swing.JButton();
        clearButton = new javax.swing.JButton();
        fakeToggleButton = new javax.swing.JToggleButton();
        jSeparator1 = new javax.swing.JToolBar.Separator();
        sendButton2 = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        devicesComboBox = new javax.swing.JComboBox();
        jLabel3 = new javax.swing.JLabel();
        countSpinner = new javax.swing.JSpinner();
        jLabel4 = new javax.swing.JLabel();
        tableSizeSpinner = new javax.swing.JSpinner();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        versionTextField = new javax.swing.JTextField();
        jLabel7 = new javax.swing.JLabel();
        ttlTextField = new javax.swing.JTextField();
        jLabel8 = new javax.swing.JLabel();
        headerLengthTextField = new javax.swing.JTextField();
        jLabel9 = new javax.swing.JLabel();
        totalLengthTextField = new javax.swing.JTextField();
        jLabel10 = new javax.swing.JLabel();
        crcTextField = new javax.swing.JTextField();
        jLabel11 = new javax.swing.JLabel();
        offsetTextField = new javax.swing.JTextField();
        reservedCheckBox = new javax.swing.JCheckBox();
        moreFragmentsCheckBox = new javax.swing.JCheckBox();
        dontFragmentCheckBox = new javax.swing.JCheckBox();
        jPanel1 = new javax.swing.JPanel();
        jLabel12 = new javax.swing.JLabel();
        phbTextField = new javax.swing.JTextField();
        jLabel13 = new javax.swing.JLabel();
        classTextField = new javax.swing.JTextField();
        jLabel14 = new javax.swing.JLabel();
        dropTextField = new javax.swing.JTextField();
        ectCheckBox = new javax.swing.JCheckBox();
        ceCheckBox = new javax.swing.JCheckBox();
        lowDelayCheckBox = new javax.swing.JCheckBox();
        highThroughputCheckBox = new javax.swing.JCheckBox();
        highReliabilityCheckBox = new javax.swing.JCheckBox();
        jLabel15 = new javax.swing.JLabel();
        precedenceTextField = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        dataTextArea = new javax.swing.JTextArea();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        exitMenuItem = new javax.swing.JMenuItem();
        jMenu2 = new javax.swing.JMenu();
        aboutMenuItem = new javax.swing.JMenuItem();

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("IP packet captor/analyzer");

        packetsTable.setModel(new javax.swing.table.DefaultTableModel(
            new Object [][] {
                {"1", "10.20.30.40", "10.20.30.50", "TCP", "200", "1024", null, new Boolean(true), "13", new Boolean(true)},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null},
                {null, null, null, null, null, null, null, null, null, null}
            },
            new String [] {
                "Identifikátor", "Odesilatel", "Příjemce", "Protokol", "Délka hlavičky", "Délka paketu", "MF", "DF", "TTL", "Options"
            }
        ) {
            Class[] types = new Class [] {
                java.lang.Object.class, java.lang.Object.class, java.lang.Object.class, java.lang.Object.class, java.lang.Object.class, java.lang.Object.class, java.lang.Boolean.class, java.lang.Boolean.class, java.lang.Object.class, java.lang.Boolean.class
            };

            public Class getColumnClass(int columnIndex) {
                return types [columnIndex];
            }
        });
        packetsTable.setSelectionMode(javax.swing.ListSelectionModel.SINGLE_SELECTION);
        packetsTable.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mouseReleased(java.awt.event.MouseEvent evt) {
                packetsTableMouseReleased(evt);
            }
        });
        packetsTable.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                packetsTableKeyReleased(evt);
            }
        });
        jScrollPane1.setViewportView(packetsTable);

        jToolBar1.setFloatable(false);
        jToolBar1.setRollover(true);

        startButton.setText("Start");
        startButton.setFocusable(false);
        startButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        startButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        startButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(startButton);

        stopButton.setText("Stop");
        stopButton.setFocusable(false);
        stopButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        stopButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        stopButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stopButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(stopButton);

        clearButton.setText("Clear");
        clearButton.setFocusable(false);
        clearButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        clearButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        clearButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(clearButton);

        fakeToggleButton.setText("Fake Options");
        fakeToggleButton.setFocusable(false);
        fakeToggleButton.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        fakeToggleButton.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        fakeToggleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                fakeToggleButtonActionPerformed(evt);
            }
        });
        jToolBar1.add(fakeToggleButton);
        jToolBar1.add(jSeparator1);

        sendButton2.setText("Send");
        sendButton2.setFocusable(false);
        sendButton2.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        sendButton2.setVerticalTextPosition(javax.swing.SwingConstants.BOTTOM);
        sendButton2.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sendButton2ActionPerformed(evt);
            }
        });
        jToolBar1.add(sendButton2);

        jLabel2.setText("Device:");

        devicesComboBox.setEnabled(false);
        devicesComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                devicesComboBoxActionPerformed(evt);
            }
        });

        jLabel3.setText("Packets count (0 for inf.):");

        countSpinner.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(0), Integer.valueOf(0), null, Integer.valueOf(1)));
        countSpinner.setValue(0);
        countSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                countSpinnerStateChanged(evt);
            }
        });

        jLabel4.setText("Only last");

        tableSizeSpinner.setModel(new javax.swing.SpinnerNumberModel(Integer.valueOf(0), Integer.valueOf(0), null, Integer.valueOf(1)));
        tableSizeSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                tableSizeSpinnerStateChanged(evt);
            }
        });

        jLabel5.setText("packets (0 for inf.)");

        jLabel6.setText("Version:");

        versionTextField.setEditable(false);

        jLabel7.setText("TTL:");

        ttlTextField.setEditable(false);

        jLabel8.setText("Header Length:");

        headerLengthTextField.setEditable(false);

        jLabel9.setText("Total Length:");

        totalLengthTextField.setEditable(false);

        jLabel10.setText("CRC:");

        crcTextField.setEditable(false);

        jLabel11.setText("Fragment Offset:");

        offsetTextField.setEditable(false);

        reservedCheckBox.setText("Reserved flag");
        reservedCheckBox.setFocusable(false);
        reservedCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                reservedCheckBoxActionPerformed(evt);
            }
        });

        moreFragmentsCheckBox.setText("More Fragments");
        moreFragmentsCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                moreFragmentsCheckBoxActionPerformed(evt);
            }
        });

        dontFragmentCheckBox.setText("Don't Fragment");
        dontFragmentCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dontFragmentCheckBoxActionPerformed(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Differentiated Services"));

        jLabel12.setText("Per-Hop Behavior:");

        phbTextField.setEditable(false);

        jLabel13.setText("Class:");

        classTextField.setEditable(false);

        jLabel14.setText("Drop:");

        dropTextField.setEditable(false);

        ectCheckBox.setText("ECN-Capable Transport");
        ectCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ectCheckBoxActionPerformed(evt);
            }
        });

        ceCheckBox.setText("ECN-CE");
        ceCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ceCheckBoxActionPerformed(evt);
            }
        });

        lowDelayCheckBox.setText("TOS Low Delay");
        lowDelayCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lowDelayCheckBoxActionPerformed(evt);
            }
        });

        highThroughputCheckBox.setText("TOS High Throughput");
        highThroughputCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                highThroughputCheckBoxActionPerformed(evt);
            }
        });

        highReliabilityCheckBox.setText("TOS High Reliability");
        highReliabilityCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                highReliabilityCheckBoxActionPerformed(evt);
            }
        });

        jLabel15.setText("TOS Precedence:");

        precedenceTextField.setEditable(false);

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel12)
                    .addComponent(jLabel13)
                    .addComponent(jLabel14))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(dropTextField)
                    .addComponent(classTextField)
                    .addComponent(phbTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 141, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(18, 18, 18)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(ectCheckBox)
                    .addComponent(ceCheckBox))
                .addGap(18, 18, 18)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(jLabel15)
                    .addComponent(precedenceTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 146, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(6, 6, 6)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(lowDelayCheckBox)
                    .addComponent(highThroughputCheckBox)
                    .addComponent(highReliabilityCheckBox))
                .addGap(31, 31, 31))
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel12)
                            .addComponent(phbTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel13)
                            .addComponent(classTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel14)
                            .addComponent(dropTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(ectCheckBox)
                            .addComponent(jLabel15)
                            .addComponent(lowDelayCheckBox))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(ceCheckBox)
                            .addComponent(precedenceTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(highThroughputCheckBox))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(highReliabilityCheckBox)))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        dataTextArea.setColumns(20);
        dataTextArea.setEditable(false);
        dataTextArea.setFont(new java.awt.Font("Monospaced", 0, 11));
        dataTextArea.setRows(5);
        jScrollPane2.setViewportView(dataTextArea);

        jMenu1.setText("File");

        exitMenuItem.setText("Exit");
        exitMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                exitMenuItemActionPerformed(evt);
            }
        });
        jMenu1.add(exitMenuItem);

        jMenuBar1.add(jMenu1);

        jMenu2.setText("Help");

        aboutMenuItem.setText("About...");
        aboutMenuItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                aboutMenuItemActionPerformed(evt);
            }
        });
        jMenu2.add(aboutMenuItem);

        jMenuBar1.add(jMenu2);

        setJMenuBar(jMenuBar1);

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jToolBar1, javax.swing.GroupLayout.DEFAULT_SIZE, 879, Short.MAX_VALUE)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel2)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(devicesComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, 224, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel3)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(countSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 54, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel4)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(tableSizeSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 49, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jLabel5)
                .addContainerGap(198, Short.MAX_VALUE))
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane1, javax.swing.GroupLayout.DEFAULT_SIZE, 859, Short.MAX_VALUE)
                .addContainerGap())
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jLabel6)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(versionTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 38, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel7)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(ttlTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 48, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel8)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(headerLengthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 48, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel9)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(totalLengthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 51, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel10)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(crcTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 51, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addGap(18, 18, 18)
                .addComponent(jLabel11)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(offsetTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 67, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addContainerGap(138, Short.MAX_VALUE))
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(reservedCheckBox)
                    .addComponent(moreFragmentsCheckBox)
                    .addComponent(dontFragmentCheckBox))
                .addGap(18, 18, 18)
                .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, 742, Short.MAX_VALUE)
                .addContainerGap())
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 859, Short.MAX_VALUE)
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addComponent(jToolBar1, javax.swing.GroupLayout.PREFERRED_SIZE, 25, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel2)
                    .addComponent(devicesComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel3)
                    .addComponent(countSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel4)
                    .addComponent(tableSizeSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel5))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, 232, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel6)
                    .addComponent(versionTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel7)
                    .addComponent(ttlTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel8)
                    .addComponent(headerLengthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel9)
                    .addComponent(totalLengthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel10)
                    .addComponent(crcTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel11)
                    .addComponent(offsetTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(reservedCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(moreFragmentsCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(dontFragmentCheckBox))
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jScrollPane2, javax.swing.GroupLayout.DEFAULT_SIZE, 193, Short.MAX_VALUE)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void startButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startButtonActionPerformed
		controller.startCapture(); //incomingToggleButton.isSelected(), outgoingToggleButton.isSelected());
}//GEN-LAST:event_startButtonActionPerformed

	private void devicesComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_devicesComboBoxActionPerformed
		controller.setDevice(devicesComboBox.getSelectedIndex());
	}//GEN-LAST:event_devicesComboBoxActionPerformed

	private void stopButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stopButtonActionPerformed
		controller.stopCapture();
	}//GEN-LAST:event_stopButtonActionPerformed

	private void countSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_countSpinnerStateChanged
		controller.setPacketsCount(Integer.parseInt(countSpinner.getValue().toString()));
	}//GEN-LAST:event_countSpinnerStateChanged

	private void tableSizeSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_tableSizeSpinnerStateChanged
		controller.setTableSize(Integer.parseInt(tableSizeSpinner.getValue().toString()));
	}//GEN-LAST:event_tableSizeSpinnerStateChanged

	private void packetsTableMouseReleased(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_packetsTableMouseReleased
		rowSelected();
	}//GEN-LAST:event_packetsTableMouseReleased

	private void packetsTableKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_packetsTableKeyReleased
		rowSelected();
	}//GEN-LAST:event_packetsTableKeyReleased

	private void sendButton2ActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sendButton2ActionPerformed
		new SenderWindowFrame(controller).setVisible(true);
	}//GEN-LAST:event_sendButton2ActionPerformed

	private void reservedCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_reservedCheckBoxActionPerformed
		reservedCheckBox.setSelected(!reservedCheckBox.isSelected());
	}//GEN-LAST:event_reservedCheckBoxActionPerformed

	private void moreFragmentsCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_moreFragmentsCheckBoxActionPerformed
		moreFragmentsCheckBox.setSelected(!moreFragmentsCheckBox.isSelected());
}//GEN-LAST:event_moreFragmentsCheckBoxActionPerformed

	private void dontFragmentCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dontFragmentCheckBoxActionPerformed
		dontFragmentCheckBox.setSelected(!dontFragmentCheckBox.isSelected());
	}//GEN-LAST:event_dontFragmentCheckBoxActionPerformed

	private void lowDelayCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lowDelayCheckBoxActionPerformed
		lowDelayCheckBox.setSelected(!lowDelayCheckBox.isSelected());
	}//GEN-LAST:event_lowDelayCheckBoxActionPerformed

	private void highThroughputCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_highThroughputCheckBoxActionPerformed
		highThroughputCheckBox.setSelected(!highThroughputCheckBox.isSelected());
	}//GEN-LAST:event_highThroughputCheckBoxActionPerformed

	private void highReliabilityCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_highReliabilityCheckBoxActionPerformed
		highReliabilityCheckBox.setSelected(!highReliabilityCheckBox.isSelected());
}//GEN-LAST:event_highReliabilityCheckBoxActionPerformed

	private void ectCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ectCheckBoxActionPerformed
		ectCheckBox.setSelected(!ectCheckBox.isSelected());
	}//GEN-LAST:event_ectCheckBoxActionPerformed

	private void ceCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ceCheckBoxActionPerformed
		ceCheckBox.setSelected(!ceCheckBox.isSelected());
	}//GEN-LAST:event_ceCheckBoxActionPerformed

	private void clearButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearButtonActionPerformed
		controller.clear();
	}//GEN-LAST:event_clearButtonActionPerformed

	private void fakeToggleButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_fakeToggleButtonActionPerformed
		controller.setFakeOptions(fakeToggleButton.isSelected());
	}//GEN-LAST:event_fakeToggleButtonActionPerformed

	private void exitMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_exitMenuItemActionPerformed
		System.exit(0);
	}//GEN-LAST:event_exitMenuItemActionPerformed

	private void aboutMenuItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_aboutMenuItemActionPerformed
		JOptionPane.showMessageDialog(this, "IP packet captor/analyzer\nSemestralni prace KIV/PSI\nJiri Kucera, A08N092P\nkalwi@students.zcu.cz",
				"About...", JOptionPane.DEFAULT_OPTION);// TODO add your handling code here:
	}//GEN-LAST:event_aboutMenuItemActionPerformed

	
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JMenuItem aboutMenuItem;
    private javax.swing.JCheckBox ceCheckBox;
    private javax.swing.JTextField classTextField;
    private javax.swing.JButton clearButton;
    private javax.swing.JSpinner countSpinner;
    private javax.swing.JTextField crcTextField;
    private javax.swing.JTextArea dataTextArea;
    private javax.swing.JComboBox devicesComboBox;
    private javax.swing.JCheckBox dontFragmentCheckBox;
    private javax.swing.JTextField dropTextField;
    private javax.swing.JCheckBox ectCheckBox;
    private javax.swing.JMenuItem exitMenuItem;
    private javax.swing.JToggleButton fakeToggleButton;
    private javax.swing.JTextField headerLengthTextField;
    private javax.swing.JCheckBox highReliabilityCheckBox;
    private javax.swing.JCheckBox highThroughputCheckBox;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JMenu jMenu2;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JToolBar.Separator jSeparator1;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JCheckBox lowDelayCheckBox;
    private javax.swing.JCheckBox moreFragmentsCheckBox;
    private javax.swing.JTextField offsetTextField;
    private javax.swing.JTable packetsTable;
    private javax.swing.JTextField phbTextField;
    private javax.swing.JTextField precedenceTextField;
    private javax.swing.JCheckBox reservedCheckBox;
    private javax.swing.JButton sendButton2;
    private javax.swing.JButton startButton;
    private javax.swing.JButton stopButton;
    private javax.swing.JSpinner tableSizeSpinner;
    private javax.swing.JTextField totalLengthTextField;
    private javax.swing.JTextField ttlTextField;
    private javax.swing.JTextField versionTextField;
    // End of variables declaration//GEN-END:variables

}
