package psi.ip.gui;

import java.util.List;
import javax.swing.event.TableModelListener;
import javax.swing.table.*;
import jpcap.packet.IPPacket;
import psi.ip.net.IPv4Packet;

/**
 * Tabulkovy model pro zobrazeni data v tabulce s pakety.
 */
public class PacketsTableModel implements TableModel {

	private final List<IPv4Packet> packets;
	private int numcols = 5;
	private String[] cnames = new String[]{"count", "ID", "Source", "Destination", "Protocol", "TTL", "H length", "P length", "More Frag", "Don't Frag", "Options"};

	PacketsTableModel(List<IPv4Packet> packets) {
		this.packets = packets;
	}

	@Override
	protected void finalize() {
	}

	@Override
	public int getColumnCount() {
		return numcols;
	}

	@Override
	public int getRowCount() {
		synchronized (packets) {
			return packets.size();
		}
	}

	@Override
	public String getColumnName(int column) {
		return cnames[column];
	}

	@Override
	public Class getColumnClass(int column) {
		switch (column) {
			case 8:
			case 9:
				return Boolean.class;

			default:
				return String.class;

		}

	}

	@Override
	public Object getValueAt(int row, int column) {
		IPv4Packet packet;
		synchronized (packets) {
			packet = packets.get(row);
		}


		switch (column) {
			case 0:
				return packet.getCount();

			case 1:
				return packet.getIdentification();

			case 2:
				return packet.getSourceAddress().toString().split("/")[1];

			case 3:
				return packet.getDestinationAddress().toString().split("/")[1];

			case 4:
				return getProtocol(packet.getProtocol());

			case 5:
				return packet.getTimeToLive();

			case 6:
				return packet.getHeaderLength();

			case 7:
				return packet.getTotalLength();

			case 8:
				return packet.getMoreFragments();

			case 9:
				return packet.getDontFragment();

			case 10:
				return packet.getOptions().length == 0 ? "none" : packet.getOptions().length;


			default:
				return "N/A";

		}

	}

	@Override
	public boolean isCellEditable(int row, int column) {
		return false;
	}

	@Override
	public void setValueAt(Object value, int row, int column) {
	}

	@Override
	public void addTableModelListener(TableModelListener l) {
	}

	@Override
	public void removeTableModelListener(TableModelListener l) {
	}
	
	private String getProtocol(int protocol) {
		switch (protocol) {
			case IPPacket.IPPROTO_ICMP:
				return "ICMP";

			case IPPacket.IPPROTO_IGMP:
					return "IGMP";

			case IPPacket.IPPROTO_IP:
				return "IP in IP";

			case IPPacket.IPPROTO_TCP:
				return "TCP";

			case IPPacket.IPPROTO_UDP:
				return "UDP";

			case IPPacket.IPPROTO_IPv6:
				return "IPv6";

			case IPPacket.IPPROTO_HOPOPT:
				return "IPv6 hop-by-hop";

			case IPPacket.IPPROTO_IPv6_Route:
				return "IPv6 routing header";

			case IPPacket.IPPROTO_IPv6_Frag:
				return "IPv6 fragment header";

			case IPPacket.IPPROTO_IPv6_ICMP:
				return "ICMP";

			case IPPacket.IPPROTO_IPv6_NoNxt:
				return "IPv6 next header";

			case IPPacket.IPPROTO_IPv6_Opts:
				return "IPv6 dest. option";

			case 103:
				return "PIMv2";

			default:
				return String.valueOf(protocol);

		}
	}

}
