package psi.ip.gui;

/**
 * Prevadi pole s daty a s volitelnymi parametry na textove retezce vhodne pro prezentaci v GUI.
 */
public class Parser {

	public static String parseData(byte[] data) {
		String out = "DATA\n====\n\n";

		if (data == null || data.length == 0) {
			out += "NONE\n";
			return out;
		}

		int rl = 0;
		byte[] row = new byte[16];

		for (int i = 0; i < data.length; i += 16) {
			rl = Math.min(16, data.length - i);

			System.arraycopy(data, i, row, 0, rl);

			String tmp = Integer.toHexString(i);
			while (tmp.length() < 4) {
				tmp = "0" + tmp;
			}
			out += tmp + "  ";

			out += byteArrayTo16Hex(row, rl);
			out += "   ";
			out += byteArrayTo16AsciiString(row, rl);
			out += "\n";

		}

		return out;
	}

	public static String parseOptions(byte[] options) {
		String out = "";

//		int pos = 0;

		out += "OPTIONS\n=======\n\n";

		if (options == null || options.length == 0) {
			out += "NONE\n";
			return out;
		}

		int length = 1;

		for (int pos = 0; pos < options.length; pos += length) {
			byte currentByte = options[pos];
			boolean copiedFlag = (currentByte & 0x80) != 0;
			int optionClass = (currentByte & 0x60) >> 5;
			int optionNumber = (currentByte & 0x1F);
			byte[] opt;

			out += "Number: " + optionNumber + "\n";

			switch (optionNumber) {
				case 0:	// End of Option list
					out += "Description: End of Option list\n";
					out += "Class: " + parseClass(optionClass, 0) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + (copiedFlag ? " (expected " + parseCopiedFlag(false) + "!)\n" : "\n");
					length = 1;
					out += "Length: " + length + "\n";
					break;

				case 1:	// No Operation
					out += "Description: No Operation\n";
					out += "Class: " + parseClass(optionClass, 0) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + (copiedFlag ? " (expected " + parseCopiedFlag(false) + "!)\n" : "\n");
					length = 1;
					out += "Length: " + length + "\n";
					break;

				case 2:	// Security
					out += "Description: Security\n";
					out += "Class: " + parseClass(optionClass, 0) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + "\n";

					length = options[pos + 1] & 0xFF;
					out +="Length: " + length + (length == 11 ? "\n" : " (expected 11!)\n");

					opt = new byte[length];
					System.arraycopy(options, pos, opt, 0, length);
					out += parseSecurity(opt);
					break;

				case 3:	// Loose Source Routing
					out += "Description: Loose Source Routing\n";
					out += "Class: " + parseClass(optionClass, 0) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + "\n";

					length = options[pos + 1] & 0xFF;
					out += "Length: " + length + "\n";

					opt = new byte[length];
					System.arraycopy(options, pos, opt, 0, length);
					out += parseSr(opt);
					break;

				case 4:	// Internet Timestamp
					out += "Description: Internet Timestamp\n";
					out += "Class: " + parseClass(optionClass, 2) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + "\n";

					length = options[pos + 1] & 0xFF;
					out += "Length: " + length + "\n";

					opt = new byte[length];
					System.arraycopy(options, pos, opt, 0, length);
					out += parseIt(opt);
					break;

				case 7:	// Record Route
					out += "Description: Record Route\n";
					out += "Class: " + parseClass(optionClass, 0) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + "\n";

					length = options[pos + 1] & 0xFF;
					out += "Length: " + length + "\n";

					opt = new byte[length];
					System.arraycopy(options, pos, opt, 0, length);
					out += parseRr(opt);
					break;

				case 8:	// Stream ID
					out += "Description: Stream ID\n";
					out += "Class: " + parseClass(optionClass, 0) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + "\n";

					length = options[pos + 1] & 0xFF;
					out += "Length: " + length + (length == 4 ? "\n" : " (expected 4!)\n");

					if (length != 4) {
						out += "Can not parse Stream ID (unexpected length)\n";
						break;
					}

					int id = (options[pos + 2] & 0xFF) << 8 | (options[pos + 3] & 0xFF);
					out += "Stream ID: " + id + "\n";
					break;

				case 9:	// Strict Source Routing
					out += "Description: Strict Source Routing\n";
					out += "Class: " + parseClass(optionClass, 0) + "\n";
					out += "Copied flag: " + parseCopiedFlag(copiedFlag) + "\n";

					length = options[pos + 1] & 0xFF;
					out += "Length: " + length + "\n";

					opt = new byte[length];
					System.arraycopy(options, pos, opt, 0, length);
					out += parseSr(opt);
					break;

				default:

			}

			out += "\n";
		}


		return out;
	}

	private static String byteArrayTo16Hex(byte[] str, int len) {
		String out = "";
		for (int i = 0; i < 16; i++) {
			if (i < len) {
				String tmp = Integer.toHexString(str[i] & 0xFF);
				if (tmp.length() < 2) {
					out += "0" + tmp;
				} else {
					out += tmp;
				}
			} else {
				out += "  ";
			}

			if (i < 15) {
				out += " ";
			}

			if (i == 7) {
				out += " ";
			}
		}

		return out;
	}

	private static String byteArrayTo16AsciiString(byte[] str, int len) {
		String out = "";

		for (int i = 0; i < 16; i++) {
			if (i < len) {
				if (str[i] >= 0 && str[i] < 33) {
					out += ".";
				} else {
					out += new String(new byte[] {str[i]});
				}
			} else {
				out += " ";
			}

			if (i == 7) {
				out += " ";
			}
		}

		return out;
	}



	private static String parseCopiedFlag(boolean flag) {
		return (flag ? "SET" : "NOT SET");
	}

	private static String parseClass(int classs, int expected) {
		String out = "";
		switch (classs) {
			case 0:
				out += "control (0)";
				break;

			case 1:
				out += "reserved (1)";
				break;

			case 2:
				out += "debugging and measurement (2)";
				break;

			case 3:
				out += "reserved (3)";
				break;

			default:
				out += "unknown (" + classs + ")";
				break;

		}

		if (classs != expected) {
			out += " (expected " + expected + "!)";
		}

		return out;
	}

	private static String parseRr(byte[] opt) {
		String out = "";
		int length = opt.length;

		if ((length - 3) % 4 != 0) {
			return "Record Route option can not be parsed (unexpected length).\n";
		}

		int pointer = opt[2] & 0xFF;

		out += "Pointer: " + pointer + "\n";

		out += "Route addresses:\n";
		for (int i = 3; i < length; i += 4) {
			out += (opt[i] & 0xff) + "." + (opt[i + 1] & 0xFF) + "." + (opt[i + 2] & 0xFF) + "." + (opt[i + 3] & 0xFF) + (pointer == i + 1 ? " <-- pointer\n" : "\n");
		}

		return out;
	}

	private static String parseIt(byte[] opt) {
		String out = "";

		int length = opt.length;

		if (length % 4 != 0) {
			return "Internet Timestamp option can not be parsed (unexpected length).\n";
		}

		int pointer = opt[2] & 0xFF;
		int overflow = (opt[3] & 0xF0) >> 4;
		int flag = opt[3] & 0x0F;

		out += "Pointer: " + pointer + "\n";
		out += "Overflow: " + overflow + "\n";
		out += "Flag: " + flag;

		int step;
		switch (flag) {
			case 0:
				out += " (timestamps only)\n";
				step = 4;
				break;

			case 1:
				out += " (addresses and all timestamps)\n";
				step = 8;
				break;

			case 3:
				out += " (addresses and address matching timestamps)\n";
				step = 8;
				break;

			default:
				out += "(unexpected value)\n";
				return out;
		}


		out += "Timestamps:\n";
		for (int i = 4; i < length; i += step) {
			if (flag > 0) {
				out += (opt[i] & 0xff) + "." + (opt[i + 1] & 0xFF) + "." + (opt[i + 2] & 0xFF) + "." + (opt[i + 3] & 0xFF) + " "; // + (pointer == i + 1 ? " <-- pointer\n" : "\n");
				out += ((opt[i + 4] & 0x7F) << 24 | (opt[i + 5] & 0xFF) << 16 | (opt[i + 6] & 0xFF) << 8 | (opt[i + 7] & 0xFF)) + " ms" + ((opt[i + 4] & 0x80) == 0x80 ? " (non-standard value)" : "");
			} else {
				out += ((opt[i] & 0x7F) << 24 | (opt[i + 1] & 0xFF) << 16 | (opt[i + 2] & 0xFF) << 8 | (opt[i + 3] & 0xFF)) + " ms" + ((opt[i] & 0x80) == 0x80 ? " (non-standard value)" : "");
			}

			out += (pointer == i + 1 ? " <-- pointer\n" : "\n");


//			byte[] baddr = new byte[4];
//			System.arraycopy(opt, i, baddr, 0, 4);
//			try {
//				InetAddress addr = InetAddress.getByAddress(baddr);
//				out += addr.toString() + (pointer == i + 1 ? " <-- pointer\n" : "\n");
//
//			} catch (UnknownHostException ex) {
//				out += "wrong address: " + (baddr[0] & 0xff) + "." + (baddr[1] & 0xFF) + "." + (baddr[2] & 0xFF) + "." + (baddr[3] & 0xFF) + "\n";
//			}

		}

		return out;
	}

	private static String parseSr(byte[] opt) {
		String out = "";

		int length = opt.length;

		if ((length - 3) % 4 != 0) {
			return "Record Route option can not be parsed (unexpected length).\n";
		}

		int pointer = opt[2] & 0xFF;

		out += "Pointer: " + pointer + "\n";
		out += "Addresses:\n";

		for (int i = 3; i < length; i += 4) {
//			byte[] baddr = new byte[4];
//			System.arraycopy(opt, i, baddr, 0, 4);
			out += (opt[i] & 0xff) + "." + (opt[i + 1] & 0xFF) + "." + (opt[i + 2] & 0xFF) + "." + (opt[i + 3] & 0xFF) + (pointer == i + 1 ? " <-- pointer\n" : "\n");
//			try {
//				InetAddress addr = InetAddress.getByAddress(baddr);
//				out += addr.toString() + (pointer == i + 1 ? " <-- pointer\n" : "\n");
//			} catch (UnknownHostException ex) {
//				out += "wrong address: " + (baddr[0] & 0xff) + "." + (baddr[1] & 0xFF) + "." + (baddr[2] & 0xFF) + "." + (baddr[3] & 0xFF) + "\n";
//			}
		}

		return out;
	}

	private static String parseSecurity(byte[] opt) {
		String out = "";

		if (opt.length != 11) {
			return "Security option can not be parsed (unexpected length).\n";
		}

		int s = (opt[2] & 0xFF) << 8 | opt[3] & 0xFF;
		int c = (opt[4] & 0xFF) << 8 | opt[5] & 0xFF;
		int h = (opt[6] & 0xFF) << 8 | opt[7] & 0xFF;
		int tcc = (opt[8] & 0xFF) << 16 | (opt[9] & 0xFF) << 8 | opt[10] & 0xFF;

		out += "Security: ";

		switch (s) {
			case 0:	// 00000000 00000000 - Unclassified
				out += "Unclassified\n";
				break;

			case 0xF135:	// 11110001 00110101 - Confidential
				out += "Confidential\n";
				break;

			case 0x789A:	// 01111000 10011010 - EFTO
				out += "EFTO\n";
				break;

			case 0xBC4D:	// 10111100 01001101 - MMMM
				out += "MMMM\n";
				break;

			case 0x5E26:	// 01011110 00100110 - PROG
				out += "PROG\n";
				break;

			case 0xAF13:	// 10101111 00010011 - Restricted
				out += "Restricted\n";
				break;

			case 0xD788:	// 11010111 10001000 - Secret
				out += "Secret\n";
				break;

			case 0x6BC5:	// 01101011 11000101 - Top Secret
				out += "Top Secret\n";
				break;

			case 0x35E2:	// 00110101 11100010 - (Reserved for future use)
			case 0x9AF1:	// 10011010 11110001 - (Reserved for future use)
			case 0x4D78:	// 01001101 01111000 - (Reserved for future use)
			case 0x24BD:	// 00100100 10111101 - (Reserved for future use)
			case 0x135E:	// 00010011 01011110 - (Reserved for future use)
			case 0x89AF:	// 10001001 10101111 - (Reserved for future use)
			case 0xC4D6:	// 11000100 11010110 - (Reserved for future use)
			case 0xE26B:	// 11100010 01101011 - (Reserved for future use)
				out += "Unused (Reserved for future use)\n";
				break;

			default:
				out += "Unknown: " + s + "\n";
		}

		out += "Compartments: ";
		if (c == 0) {
			out += "Not compartmented\n";
		} else {
			out += "Compartmented (0x" + Integer.toHexString(c) + " - value defined by Defense Intelligence Agency)\n";
		}

		out += "Handling Restrictions: ";
		out += "0x" + Integer.toHexString(h) + " - value defined by Defense Intelligence Agency\n";

		out += "Transmission Control Code: 0x" + Integer.toHexString(tcc) + "\n";

		return out;
	}
}
