package psi.ip.gui;

import java.net.InetAddress;
import java.net.UnknownHostException;
import javax.swing.JOptionPane;
import javax.swing.SpinnerNumberModel;
import jpcap.packet.IPPacket;

/**
 * Okno pro odesilani paketu.
 */
public class SenderWindowFrame extends javax.swing.JFrame {

	private Controller controller;
	private int tos = 0;

	/**
	 * Konstruktor vytvori instanci tridy.
	 * @param controller Rozhrani mezi sitovou a prezentacni vrstvou.
	 */
	public SenderWindowFrame(Controller controller) {
		this.controller = controller;

		initComponents();
		srcMacTextField.setText(macToString(controller.getCurrentDeviceMacAddress()));
		setupDs();
		setTos();
		protocolTextField.setText(String.valueOf(getComboBoxProtocol()));
	}

	/**
	 * Prevede pole s MAC adresou na citelny retezec.
	 * @param mac MAC adresa zadana jako sestice bytu.
	 * @return Retezec s MAC adresou.
	 */
	private static String macToString(byte[] mac) {
		String out = "";

		for (int i = 0; i < mac.length; i++) {

			String tmp = Integer.toString((mac[i] & 0xff) + 0x100, 16).substring(1).toUpperCase();
			while (tmp.length() < 2) {
				tmp = "0" + tmp;
			}
			out += tmp;
			if (i < mac.length - 1) {
				out += "-";
			}
		}

		return out;
	}

	/**
	 * Prevede retezec s MAC adresou na pole.
	 * @param str Retezec s MAC adresou, cisla jsou zadana hexadecimalne,
	 * oddelovac je dvojtecka nebo pomlcka.
	 * @return Sestice bytu s MAC adresou.
	 * @throws java.lang.Exception
	 */
	private static byte[] stringToMac(String str) throws Exception {
		byte[] out = new byte[6];
		String[] tmp = str.split("[:-]");

		if (tmp.length != 6) {
			throw new Exception("Invalid MAC address");
		}
		
		for (int i = 0; i < 6; i++) {
			out[i] = (byte) Integer.valueOf(tmp[i], 16).intValue();
		}
		return out;

	}

	/**
	 * Odesle paket.
	 */
	private void sendPacket() {
		IPPacket packet = new IPPacket();

		int ident;
		try {
			ident = Integer.valueOf(identifiactionTextField.getText());
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(this, "Identification must be a number", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

		int offset;
		try {
			offset = Integer.valueOf(offsetTextField.getText());
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(this, "Offset must be a number", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

		int ttl;
		try {
			ttl = Integer.valueOf(ttlTextField.getText());
		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(this, "Time To Live must be a number", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

		int protocol;
		try {

			if (protocolTextField.isEnabled()) {
				protocol = Integer.valueOf(protocolTextField.getText());
			} else {
				protocol = getComboBoxProtocol();
			}

		} catch (NumberFormatException e) {
			JOptionPane.showMessageDialog(this, "Protocol must be a number", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

		InetAddress src;
		try {
			src = getAddressFromString(srcTextField.getText());
		} catch (UnknownHostException ex) {
			JOptionPane.showMessageDialog(this, "Unknown source address", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

		InetAddress dest;
		try {
			dest = getAddressFromString(destTextField.getText());
		} catch (UnknownHostException ex) {
			JOptionPane.showMessageDialog(this, "Unknown destination address", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

		packet.setIPv4Parameter(0, // priority
				false, // d_flag
				false, // t_flag
				false, // r_flag
				tos, // rsv_tos
				reservedCheckBox.isSelected(), // rsv_frag
				dontFragmentCheckBox.isSelected(), // dont_frag
				moreFragmentsCheckBox.isSelected(), // more_frag
				offset, // offset
				ident, // ident
				ttl, // ttl
				protocol, // protocol
				src, // src
				dest // dst
				);	//

		packet.version = 4;
		packet.option = new byte[]{1, 2, 3, 0};

		packet.length = 100;

		packet.data = dataTextArea.getText().getBytes();
		packet.header = dataTextArea.getText().getBytes();

		byte[] srcMac;
		try {
			srcMac = stringToMac(srcMacTextField.getText());
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Invalid source MAC address", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

		byte[] destMac;
		try {
			destMac = stringToMac(destMacTextField.getText());
		} catch (Exception e) {
			JOptionPane.showMessageDialog(this, "Invalid destination MAC address", "Bad value", JOptionPane.ERROR_MESSAGE);
			return;
		}

//		controller.sendPacket(packet, new byte[]{(byte) 0, (byte) 0x0E, (byte) 0x0C, (byte) 0xD9, (byte) 0xB5, (byte) 0xC8}, new byte[]{(byte) 0, (byte) 0x19, (byte) 0xe8, (byte) 0x3b, (byte) 0xf7, (byte) 0xc3});
		controller.sendPacket(packet, srcMac, destMac);
	}

	/**
	 * Vrati cislo zvoleneho protokolu.
	 * @return Cislo protokolu.
	 */
	private int getComboBoxProtocol() {
		int protocol;
		switch (protocolComboBox.getSelectedIndex()) {
			case 0: // ICMP
				protocol = IPPacket.IPPROTO_ICMP;
				break;

			case 1: // IGMP
				protocol = IPPacket.IPPROTO_IGMP;
				break;

			case 2: // TCP
				protocol = IPPacket.IPPROTO_TCP;
				break;

			case 3: // UDP
				protocol = IPPacket.IPPROTO_UDP;
				break;

			case 4: // OSPF
				protocol = 89;
				break;

			case 5: // SCTP
				protocol = 132;
				break;

			default:
				protocol = 0;

		}
		return protocol;
	}

	/**
	 * Prevede IP adresu zadanou retezcem na objekt typu <code>InetAddress</code>
	 * @param str IP adresa zadana retezcem, oddelovacem mezi cisly jsou tecky.
	 * @return Objekt typu <code>InetAddress</code>.
	 * @throws java.net.UnknownHostException
	 */
	private InetAddress getAddressFromString(String str) throws UnknownHostException {
		String[] spl = str.split(".");
		if (spl.length == 4) {
			try {
				byte[] ip = new byte[]{Byte.parseByte(spl[0]), Byte.parseByte(spl[1]), Byte.parseByte(spl[2]), Byte.parseByte(spl[3])};
				return InetAddress.getByAddress(ip);
			} catch (NumberFormatException e) {
				// nothing
			}
		}

		return InetAddress.getByName(str);

	}

	/**
	 * Nastavi dalsi moznosti Differentiated Services podle zvoleneho PHB.
	 */
	private void setupDs() {
		switch (phbComboBox.getSelectedIndex()) {
			case 0:	// Default
				classSpinner.setEnabled(false);
				dropComboBox.setEnabled(false);
				break;

			case 1:	// EF
				classSpinner.setEnabled(false);
				dropComboBox.setEnabled(false);
				break;

			case 2:	// AF
				classSpinner.setEnabled(true);
				classSpinner.setModel(new SpinnerNumberModel(Math.min((Integer) classSpinner.getValue(), 4), 1, 4, 1));
				dropComboBox.setEnabled(true);
				break;

			case 3:	// CS
				classSpinner.setEnabled(true);
				classSpinner.setModel(new SpinnerNumberModel(((Integer) classSpinner.getValue()).intValue(), 1, 7, 1));
				dropComboBox.setEnabled(false);
				break;

		}
	}

	/**
	 * Nastavi zobrazeni osmice bitu pole TOS.
	 */
	private void setTos() {
		switch (phbComboBox.getSelectedIndex()) {
			case 0:	// Default
				tos = 0;
				break;

			case 1:	// EF
				tos = 0xB8;
				break;

			case 2:	// AF
				tos = (Integer) classSpinner.getValue() << 5;
				tos = tos | (dropComboBox.getSelectedIndex() + 1) << 3;
				break;

			case 3:	// CS
				tos = (Integer) classSpinner.getValue() << 5;
				break;

		}

		tos = tos | (ectCheckBox.isSelected() ? 2 : 0);
		tos = tos | (ceCheckBox.isSelected() ? 1 : 0);


		ds0TextField.setText((tos & (1 << 7)) == 0 ? "0" : "1");
		ds1TextField.setText((tos & (1 << 6)) == 0 ? "0" : "1");
		ds2TextField.setText((tos & (1 << 5)) == 0 ? "0" : "1");
		ds3TextField.setText((tos & (1 << 4)) == 0 ? "0" : "1");
		ds4TextField.setText((tos & (1 << 3)) == 0 ? "0" : "1");
		ds5TextField.setText((tos & (1 << 2)) == 0 ? "0" : "1");
		ds6TextField.setText((tos & (1 << 1)) == 0 ? "0" : "1");
		ds7TextField.setText((tos & (1 << 0)) == 0 ? "0" : "1");
	}

    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jLabel1 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        jLabel7 = new javax.swing.JLabel();
        jLabel8 = new javax.swing.JLabel();
        jLabel9 = new javax.swing.JLabel();
        jLabel11 = new javax.swing.JLabel();
        jLabel12 = new javax.swing.JLabel();
        versionTextField = new javax.swing.JTextField();
        ihlTextField = new javax.swing.JTextField();
        totalLengthTextField = new javax.swing.JTextField();
        identifiactionTextField = new javax.swing.JTextField();
        offsetTextField = new javax.swing.JTextField();
        ttlTextField = new javax.swing.JTextField();
        protocolTextField = new javax.swing.JTextField();
        srcTextField = new javax.swing.JTextField();
        destTextField = new javax.swing.JTextField();
        sendButton = new javax.swing.JButton();
        jPanel1 = new javax.swing.JPanel();
        jLabel13 = new javax.swing.JLabel();
        phbComboBox = new javax.swing.JComboBox();
        jLabel2 = new javax.swing.JLabel();
        classSpinner = new javax.swing.JSpinner();
        jLabel14 = new javax.swing.JLabel();
        dropComboBox = new javax.swing.JComboBox();
        jLabel15 = new javax.swing.JLabel();
        ectCheckBox = new javax.swing.JCheckBox();
        ceCheckBox = new javax.swing.JCheckBox();
        ds0TextField = new javax.swing.JTextField();
        ds1TextField = new javax.swing.JTextField();
        ds2TextField = new javax.swing.JTextField();
        ds3TextField = new javax.swing.JTextField();
        ds4TextField = new javax.swing.JTextField();
        ds5TextField = new javax.swing.JTextField();
        ds6TextField = new javax.swing.JTextField();
        ds7TextField = new javax.swing.JTextField();
        jLabel16 = new javax.swing.JLabel();
        reservedCheckBox = new javax.swing.JCheckBox();
        dontFragmentCheckBox = new javax.swing.JCheckBox();
        moreFragmentsCheckBox = new javax.swing.JCheckBox();
        protocolComboBox = new javax.swing.JComboBox();
        jLabel17 = new javax.swing.JLabel();
        jLabel10 = new javax.swing.JLabel();
        jScrollPane1 = new javax.swing.JScrollPane();
        dataTextArea = new javax.swing.JTextArea();
        jLabel18 = new javax.swing.JLabel();
        jLabel19 = new javax.swing.JLabel();
        srcMacTextField = new javax.swing.JTextField();
        jLabel20 = new javax.swing.JLabel();
        destMacTextField = new javax.swing.JTextField();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Send packet");
        setResizable(false);

        jLabel1.setText("Version:");

        jLabel3.setText("Header length:");

        jLabel4.setText("Total length:");

        jLabel5.setText("Identification:");

        jLabel6.setText("Flags:");

        jLabel7.setText("Fragment offset:");

        jLabel8.setText("TTL:");

        jLabel9.setText("Protocol:");

        jLabel11.setText("Source address:");

        jLabel12.setText("Destination address:");

        versionTextField.setEditable(false);
        versionTextField.setText("4");

        ihlTextField.setEditable(false);
        ihlTextField.setText("20");

        totalLengthTextField.setEditable(false);
        totalLengthTextField.setText("20");

        identifiactionTextField.setText("12345");

        offsetTextField.setEditable(false);
        offsetTextField.setText("0");
        offsetTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                offsetTextFieldActionPerformed(evt);
            }
        });

        ttlTextField.setText("64");

        protocolTextField.setText("0");
        protocolTextField.setEnabled(false);

        srcTextField.setColumns(2);
        srcTextField.setText("localhost");

        destTextField.setText("localhost");

        sendButton.setText("Send");
        sendButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sendButtonActionPerformed(evt);
            }
        });

        jPanel1.setBorder(javax.swing.BorderFactory.createTitledBorder("Differentiated Services"));

        jLabel13.setText("Per-Hop Behavior:");

        phbComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Default", "Expedited Forwarding", "Assured Forwarding", "Class Selector" }));
        phbComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                phbComboBoxActionPerformed(evt);
            }
        });

        jLabel2.setText("Class:");

        classSpinner.setModel(new javax.swing.SpinnerNumberModel(1, 1, 7, 1));
        classSpinner.setEnabled(false);
        classSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                classSpinnerStateChanged(evt);
            }
        });

        jLabel14.setText("Drop:");

        dropComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Low", "Medium", "High" }));
        dropComboBox.setEnabled(false);
        dropComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dropComboBoxActionPerformed(evt);
            }
        });

        jLabel15.setText("Explicit Congestion Notification:");

        ectCheckBox.setText("ECN-Capable Transport");
        ectCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ectCheckBoxActionPerformed(evt);
            }
        });

        ceCheckBox.setText("ECN-CE bit");
        ceCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                ceCheckBoxActionPerformed(evt);
            }
        });

        ds0TextField.setEditable(false);
        ds0TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds0TextField.setText("0");

        ds1TextField.setEditable(false);
        ds1TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds1TextField.setText("0");

        ds2TextField.setEditable(false);
        ds2TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds2TextField.setText("0");

        ds3TextField.setEditable(false);
        ds3TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds3TextField.setText("0");

        ds4TextField.setEditable(false);
        ds4TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds4TextField.setText("0");

        ds5TextField.setEditable(false);
        ds5TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds5TextField.setText("0");

        ds6TextField.setEditable(false);
        ds6TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds6TextField.setText("0");

        ds7TextField.setEditable(false);
        ds7TextField.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        ds7TextField.setText("0");

        jLabel16.setText("Differentiated Services Field:");

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addComponent(ceCheckBox)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel14)
                                    .addComponent(jLabel2))
                                .addGap(67, 67, 67))
                            .addGroup(jPanel1Layout.createSequentialGroup()
                                .addComponent(jLabel13)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)))
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(dropComboBox, 0, 194, Short.MAX_VALUE)
                            .addComponent(classSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, 56, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(phbComboBox, 0, 194, Short.MAX_VALUE)))
                    .addComponent(jLabel15)
                    .addComponent(ectCheckBox)
                    .addComponent(jLabel16)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addComponent(ds0TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ds1TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ds2TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ds3TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(18, 18, 18)
                        .addComponent(ds4TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ds5TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ds6TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(ds7TextField, javax.swing.GroupLayout.PREFERRED_SIZE, 20, javax.swing.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(phbComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel13))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(classSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel14)
                    .addComponent(dropComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel15)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(ectCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(ceCheckBox)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel16)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(ds0TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ds1TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ds2TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ds3TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ds4TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ds5TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ds6TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(ds7TextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );

        reservedCheckBox.setText("Reserved");

        dontFragmentCheckBox.setText("Don't Fragment");

        moreFragmentsCheckBox.setText("More Fragments");

        protocolComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "ICMP", "IGMP", "TCP", "UDP", "OSPF", "SCTP", "Other" }));
        protocolComboBox.setSelectedIndex(2);
        protocolComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                protocolComboBoxActionPerformed(evt);
            }
        });

        jLabel17.setText("Protocol number:");

        jLabel10.setText("Data:");

        dataTextArea.setColumns(20);
        dataTextArea.setFont(new java.awt.Font("Tahoma", 0, 11));
        dataTextArea.setRows(5);
        dataTextArea.addKeyListener(new java.awt.event.KeyAdapter() {
            public void keyReleased(java.awt.event.KeyEvent evt) {
                dataTextAreaKeyReleased(evt);
            }
        });
        jScrollPane1.setViewportView(dataTextArea);

        jLabel18.setText("Override Ethernet MAC addresses:");

        jLabel19.setText("Source address:");

        srcMacTextField.setText("jTextField1");

        jLabel20.setText("Destination address:");

        destMacTextField.setText("00-11-22-33-44-55");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(javax.swing.GroupLayout.Alignment.TRAILING, layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabel4)
                            .addComponent(jLabel5)
                            .addComponent(jLabel6)
                            .addGroup(layout.createSequentialGroup()
                                .addComponent(jLabel8)
                                .addGap(205, 205, 205))
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel1)
                                    .addComponent(jLabel3)
                                    .addComponent(jLabel7)
                                    .addComponent(jLabel10))
                                .addGap(27, 27, 27)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(versionTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)
                                    .addComponent(reservedCheckBox)
                                    .addComponent(identifiactionTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)
                                    .addComponent(totalLengthTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)
                                    .addComponent(ihlTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)
                                    .addComponent(dontFragmentCheckBox)
                                    .addComponent(moreFragmentsCheckBox)
                                    .addComponent(offsetTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)
                                    .addComponent(ttlTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE)
                                    .addComponent(protocolComboBox, 0, 117, Short.MAX_VALUE)
                                    .addComponent(srcTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 117, Short.MAX_VALUE))))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addGroup(layout.createSequentialGroup()
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                                    .addComponent(jLabel12)
                                    .addComponent(jLabel17))
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                                    .addComponent(protocolTextField)
                                    .addComponent(destTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 123, Short.MAX_VALUE)))
                            .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(jLabel9)
                    .addComponent(jLabel11)
                    .addComponent(sendButton, javax.swing.GroupLayout.Alignment.TRAILING)
                    .addComponent(jScrollPane1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 560, Short.MAX_VALUE)
                    .addComponent(jLabel18)
                    .addGroup(layout.createSequentialGroup()
                        .addComponent(jLabel19)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(srcMacTextField, javax.swing.GroupLayout.PREFERRED_SIZE, 158, javax.swing.GroupLayout.PREFERRED_SIZE)
                        .addGap(16, 16, 16)
                        .addComponent(jLabel20)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(destMacTextField, javax.swing.GroupLayout.DEFAULT_SIZE, 201, Short.MAX_VALUE)))
                .addContainerGap())
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(layout.createSequentialGroup()
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel1)
                            .addComponent(versionTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel3)
                            .addComponent(ihlTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel4)
                            .addComponent(totalLengthTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel5)
                            .addComponent(identifiactionTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel6)
                            .addComponent(reservedCheckBox))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(dontFragmentCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(moreFragmentsCheckBox)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(offsetTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addComponent(jLabel7))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                        .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabel8)
                            .addComponent(ttlTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(jPanel1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addGap(9, 9, 9)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel9)
                    .addComponent(protocolComboBox, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel17)
                    .addComponent(protocolTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel11)
                    .addComponent(srcTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(jLabel12)
                    .addComponent(destTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel10)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(jScrollPane1, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
                .addComponent(jLabel18)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(jLabel19)
                    .addComponent(jLabel20)
                    .addComponent(srcMacTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(destMacTextField, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED, 27, Short.MAX_VALUE)
                .addComponent(sendButton)
                .addContainerGap())
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

	private void sendButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sendButtonActionPerformed
		sendPacket();
}//GEN-LAST:event_sendButtonActionPerformed

	private void phbComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_phbComboBoxActionPerformed
		setupDs();
		setTos();
	}//GEN-LAST:event_phbComboBoxActionPerformed

	private void classSpinnerStateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_classSpinnerStateChanged
		setTos();
	}//GEN-LAST:event_classSpinnerStateChanged

	private void dropComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dropComboBoxActionPerformed
		setTos();
	}//GEN-LAST:event_dropComboBoxActionPerformed

	private void ectCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ectCheckBoxActionPerformed
		setTos();
	}//GEN-LAST:event_ectCheckBoxActionPerformed

	private void ceCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_ceCheckBoxActionPerformed
		setTos();
	}//GEN-LAST:event_ceCheckBoxActionPerformed

	private void protocolComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_protocolComboBoxActionPerformed
		protocolTextField.setEnabled(protocolComboBox.getSelectedIndex() == 6);
		protocolTextField.setText(String.valueOf(getComboBoxProtocol()));
	}//GEN-LAST:event_protocolComboBoxActionPerformed

	private void dataTextAreaKeyReleased(java.awt.event.KeyEvent evt) {//GEN-FIRST:event_dataTextAreaKeyReleased
		totalLengthTextField.setText(String.valueOf(20 + dataTextArea.getText().length()));
	}//GEN-LAST:event_dataTextAreaKeyReleased

	private void offsetTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_offsetTextFieldActionPerformed
		// TODO add your handling code here:
	}//GEN-LAST:event_offsetTextFieldActionPerformed

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JCheckBox ceCheckBox;
    private javax.swing.JSpinner classSpinner;
    private javax.swing.JTextArea dataTextArea;
    private javax.swing.JTextField destMacTextField;
    private javax.swing.JTextField destTextField;
    private javax.swing.JCheckBox dontFragmentCheckBox;
    private javax.swing.JComboBox dropComboBox;
    private javax.swing.JTextField ds0TextField;
    private javax.swing.JTextField ds1TextField;
    private javax.swing.JTextField ds2TextField;
    private javax.swing.JTextField ds3TextField;
    private javax.swing.JTextField ds4TextField;
    private javax.swing.JTextField ds5TextField;
    private javax.swing.JTextField ds6TextField;
    private javax.swing.JTextField ds7TextField;
    private javax.swing.JCheckBox ectCheckBox;
    private javax.swing.JTextField identifiactionTextField;
    private javax.swing.JTextField ihlTextField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel10;
    private javax.swing.JLabel jLabel11;
    private javax.swing.JLabel jLabel12;
    private javax.swing.JLabel jLabel13;
    private javax.swing.JLabel jLabel14;
    private javax.swing.JLabel jLabel15;
    private javax.swing.JLabel jLabel16;
    private javax.swing.JLabel jLabel17;
    private javax.swing.JLabel jLabel18;
    private javax.swing.JLabel jLabel19;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel20;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel7;
    private javax.swing.JLabel jLabel8;
    private javax.swing.JLabel jLabel9;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JCheckBox moreFragmentsCheckBox;
    private javax.swing.JTextField offsetTextField;
    private javax.swing.JComboBox phbComboBox;
    private javax.swing.JComboBox protocolComboBox;
    private javax.swing.JTextField protocolTextField;
    private javax.swing.JCheckBox reservedCheckBox;
    private javax.swing.JButton sendButton;
    private javax.swing.JTextField srcMacTextField;
    private javax.swing.JTextField srcTextField;
    private javax.swing.JTextField totalLengthTextField;
    private javax.swing.JTextField ttlTextField;
    private javax.swing.JTextField versionTextField;
    // End of variables declaration//GEN-END:variables
}
