package psi.ip.net;

import java.io.IOException;
import java.util.Observable;
import java.util.logging.Level;
import java.util.logging.Logger;
import jpcap.*;
import jpcap.packet.*;
import psi.ip.gui.Controller;

/**
 * Trida pro zachytavani paketu.
 */
public class Captor extends Observable implements Runnable {

	private NetworkInterface device;
	private Thread thread;
	private volatile boolean running = true;

	/**
	 * Nastavi sitove rozhrani pouzite pro zachytavani.
	 * @param device Sitove rozhrani pouzite pro zachytavani.
	 */
	public void setDevice(NetworkInterface device) {
		this.device = device;
	}

	/**
	 * Spusti zachytavani.
	 */
	public void start() {

		running = true;
		thread = new Thread(this);
		thread.start();
	}

	/**
	 * Zastavi zachytavani.
	 */
	public void stop() {
		running = false;
	}

	@Override
	public void run() {
		JpcapCaptor captor;

		try {
			captor = JpcapCaptor.openDevice(device, 2000, true, 1000);
			captor.setFilter("ip", true);
		} catch (IOException ex) {
			Logger.getLogger(Captor.class.getName()).log(Level.SEVERE, null, ex);
			setChanged();
			notifyObservers(Controller.MSG_CAPTURE_STOPPED);
			return;
		}
		

		while (true) {
			Packet packet = captor.getPacket();
			if (!running) {
				break;
			}
			
			try {
				IPPacket ipPacket = (IPPacket) packet;
				setChanged();
				notifyObservers(ipPacket);
			} catch (NullPointerException e) {
				// vyprsel timeout, nic se nedeje, ceka se znova
			}

		}

		setChanged();
		notifyObservers(Controller.MSG_CAPTURE_STOPPED);

	}
}
