package psi.ip.net;

/**
 * Generator falesnych volitelnych parametru.
 */
public class FakeOptionsGenerator {

	private static int counter = 0;

	private static final byte[] NOP = new byte[] {
		(byte) 1, (byte) 0, (byte) 0, (byte) 0
	};

	private static final byte[] SECURITY = new byte[]{
		(byte) 0x82, (byte) 0x0B, (byte) 0x6B, (byte) 0xC5,	// type, length (11), S (top security)
		(byte) 0x10, (byte) 0x20, (byte) 0x30, (byte) 0x40,	// C, H
		(byte) 0x50, (byte) 0x60, (byte) 0x70, (byte) 0x00	// TCC, EOOL
	};

	private static final byte[] LSRR = new byte[]{
		(byte) 0x83, (byte) 15, (byte) 8, // type, length, pointer
		(byte) 192, (byte) 168, (byte) 0, (byte) 1,
		(byte) 192, (byte) 168, (byte) 0, (byte) 2,
		(byte) 192, (byte) 168, (byte) 0, (byte) 3,
		(byte) 0
	};

	private static final byte[] SSRR = new byte[] {
		(byte) 0x89, (byte) 19, (byte) 12, // type, length, pointer
		(byte) 10, (byte) 0, (byte) 0, (byte) 4,
		(byte) 10, (byte) 0, (byte) 0, (byte) 5,
		(byte) 10, (byte) 0, (byte) 0, (byte) 6,
		(byte) 10, (byte) 0, (byte) 0, (byte) 6,
		(byte) 0
	};

	private static final byte[] RR = new byte[] {
		(byte) 0x07, (byte) 11, (byte) 12, // type, length, pointer
		(byte) 172, (byte) 16, (byte) 0, (byte) 6,
		(byte) 172, (byte) 16, (byte) 0, (byte) 7,
		(byte) 0
	};

	private static final byte[] SI = new byte[] {
		(byte) 0x88, (byte) 4, (byte) 10, (byte) 20,	// type, length, ID (2x)
	};

	private static final byte[] IT = new byte[] {
		(byte) 0x44, (byte) 28, (byte) 5, (byte) 3,	//type, length, pointer, flags
		(byte) 10, (byte) 20, (byte) 30, (byte) 40,	// IP
		(byte) 1, (byte) 1, (byte) 0, (byte) 10,	// TS
		(byte) 10, (byte) 20, (byte) 30, (byte) 50,
		(byte) 1, (byte) 1, (byte) 0, (byte) 70,
		(byte) 10, (byte) 20, (byte) 30, (byte) 60,
		(byte) 128, (byte) 1, (byte) 0, (byte) 10
		
	};

	/**
	 * Vrati falesny volitelny parametr.
	 * @return falesny volitelny parametr.
	 */
	public static byte[] getOptions() {
		switch (counter++ % 7) {
			case 0:
				return NOP;

			case 1:
				return SECURITY;

			case 2:
				return LSRR;

			case 3:
				return SSRR;

			case 4:
				return RR;

			case 5:
				return SI;

			case 6:
				return IT;

			default:
				return NOP;
		}
	}

}
