package psi.ip.net;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;
import jpcap.JpcapSender;
import jpcap.NetworkInterface;
import jpcap.packet.EthernetPacket;
import jpcap.packet.Packet;

/**
 * Trida pro odesilani paketu.
 */
public class Sender {

	private NetworkInterface device;

	/**
	 * Nastavi sitove rozhrani, na ktere se budou odesilat pakety.
	 * @param device sitove rozhrani, na ktere se budou odesilat pakety.
	 */
	public void setDevice(NetworkInterface device) {
		this.device = device;
	}

	/**
	 * Odesle IP paket, Ethernetovemu ramci nastavi zadanou zdrojovou a cilovou MAC adresu.
	 * @param packet Paket k odeslani.
	 * @param srcMac Zdrojova MAC adresa pro Ethernet ramec.
	 * @param destMac Cilova MAC adresa pro Ethernet ramec.
	 */
	public void sendPacket(Packet packet, byte[] srcMac, byte destMac[]) {
		JpcapSender sender;
		try {
			sender = JpcapSender.openDevice(device);
		} catch (IOException ex) {
			Logger.getLogger(Sender.class.getName()).log(Level.SEVERE, null, ex);
			return;
		}

		EthernetPacket ether = new EthernetPacket();
		ether.frametype = EthernetPacket.ETHERTYPE_IP;
		ether.src_mac = srcMac;
		ether.dst_mac = destMac;
		packet.datalink = ether;

		sender.sendPacket(packet);
		
	}
}
